=== modified file 'CMakeLists.txt'
--- CMakeLists.txt	2012-11-01 15:47:16 +0000
+++ CMakeLists.txt	2012-11-06 12:08:10 +0000
@@ -121,6 +121,7 @@
   ${GTEST_INCLUDE_DIR}
 )
 
+add_subdirectory(benchmarks/)
 add_subdirectory(tests/)
 add_subdirectory(tools/)
 add_subdirectory(guides/)

=== added directory 'benchmarks'
=== added file 'benchmarks/CMakeLists.txt'
--- benchmarks/CMakeLists.txt	1970-01-01 00:00:00 +0000
+++ benchmarks/CMakeLists.txt	2012-11-06 12:08:10 +0000
@@ -0,0 +1,24 @@
+# Copyright © 2012 Canonical Ltd.
+#
+# This program is free software: you can redistribute it and/or modify
+# it under the terms of the GNU General Public License version 3 as
+# published by the Free Software Foundation.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+#
+# Authored by: Thomas Voss <thomas.voss@canonical.com>
+
+add_custom_target(
+  benchmarks
+)
+
+add_subdirectory(cpu)
+add_subdirectory(memory)
+
+add_dependencies(benchmarks cpu_benchmarks memory_benchmarks)
\ No newline at end of file

=== added directory 'benchmarks/cpu'
=== added file 'benchmarks/cpu/CMakeLists.txt'
--- benchmarks/cpu/CMakeLists.txt	1970-01-01 00:00:00 +0000
+++ benchmarks/cpu/CMakeLists.txt	2012-11-06 12:08:10 +0000
@@ -0,0 +1,67 @@
+# Copyright © 2012 Canonical Ltd.
+#
+# This program is free software: you can redistribute it and/or modify
+# it under the terms of the GNU General Public License version 3 as
+# published by the Free Software Foundation.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+#
+# Authored by: Thomas Voss <thomas.voss@canonical.com>
+
+find_program(
+  VALGRIND_EXECUTABLE
+  valgrind
+)
+
+if(NOT VALGRIND_EXECUTABLE)
+  message("valgrind not found, disabling cpu benchmarks")
+else()
+  set(CPU_BENCHMARKS_SOCKET "/tmp/benchmarks.cpu.socket.mir")
+
+  configure_file(
+    ${CMAKE_CURRENT_SOURCE_DIR}/benchmark.sh.in
+    ${CMAKE_CURRENT_BINARY_DIR}/benchmark.sh
+    )
+
+  add_custom_target(
+    cpu_benchmark_one_server_one_client
+    "${CMAKE_CURRENT_BINARY_DIR}/benchmark.sh"
+    1
+    1000
+    one_server_one_client_results_for_client
+    one_server_one_client_results_for_server
+    )
+
+  add_custom_target(
+    cpu_benchmark_one_server_multiple_clients
+    "${CMAKE_CURRENT_BINARY_DIR}/benchmark.sh"
+    10
+    100
+    one_server_multiple_client_results_for_client
+    one_server_multiple_client_results_for_server
+    )
+
+  add_custom_target(
+    cpu_benchmark_one_server_multiple_clients_heavy_load
+    "${CMAKE_CURRENT_BINARY_DIR}/benchmark.sh"
+    20
+    100
+    one_server_multiple_client_heavy_load_results_for_client
+    one_server_multiple_client_heavy_load_results_for_server
+    )
+
+  add_custom_target(
+    cpu_benchmarks
+    DEPENDS
+    cpu_benchmark_one_server_one_client
+    cpu_benchmark_one_server_multiple_clients
+    cpu_benchmark_one_server_multiple_clients_heavy_load
+    )
+
+endif()

=== added file 'benchmarks/cpu/benchmark.sh.in'
--- benchmarks/cpu/benchmark.sh.in	1970-01-01 00:00:00 +0000
+++ benchmarks/cpu/benchmark.sh.in	2012-11-06 12:08:10 +0000
@@ -0,0 +1,18 @@
+#!/bin/bash
+
+# Starting the server
+server_fn=@CMAKE_CURRENT_BINARY_DIR@/callgrind_$4
+@VALGRIND_EXECUTABLE@ --tool=callgrind --callgrind-out-file=`echo $server_fn`.out @CMAKE_INSTALL_PREFIX@/bin/mir -f @CPU_BENCHMARKS_SOCKET@ &
+server_pid=$!
+
+sleep 5
+
+# Starting the clients
+fn=@CMAKE_CURRENT_BINARY_DIR@/callgrind_$3
+seq 1 $1 | xargs -I {} -n 1 -P $1 @VALGRIND_EXECUTABLE@ --tool=callgrind --callgrind-out-file=`echo $fn`.out.{} @CMAKE_INSTALL_PREFIX@/bin/mir_demo_client -f @CPU_BENCHMARKS_SOCKET@ -c $2
+
+kill $server_pid
+
+# This is extremely ugly, but we need to introduce some latency to account
+# for the fact that we fire up multiple mir instances in rapid succession.
+sleep 5

=== added directory 'benchmarks/memory'
=== added file 'benchmarks/memory/CMakeLists.txt'
--- benchmarks/memory/CMakeLists.txt	1970-01-01 00:00:00 +0000
+++ benchmarks/memory/CMakeLists.txt	2012-11-06 12:08:10 +0000
@@ -0,0 +1,68 @@
+# Copyright © 2012 Canonical Ltd.
+#
+# This program is free software: you can redistribute it and/or modify
+# it under the terms of the GNU General Public License version 3 as
+# published by the Free Software Foundation.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+#
+# Authored by: Thomas Voss <thomas.voss@canonical.com>
+
+find_program(
+  VALGRIND_EXECUTABLE
+  valgrind
+)
+
+if(NOT VALGRIND_EXECUTABLE)
+  message("valgrind not found, disabling memory benchmarks")
+else()
+
+  set(MEMORY_BENCHMARKS_SOCKET "/tmp/benchmarks.memory.socket.mir")
+
+  configure_file(
+    ${CMAKE_CURRENT_SOURCE_DIR}/benchmark.sh.in
+    ${CMAKE_CURRENT_BINARY_DIR}/benchmark.sh
+    )
+
+  add_custom_target(
+    memory_benchmark_one_server_one_client
+    "${CMAKE_CURRENT_BINARY_DIR}/benchmark.sh"
+    1
+    100
+    one_server_one_client_results_for_client
+    one_server_one_client_results_for_server
+    )
+
+  add_custom_target(
+    memory_benchmark_one_server_multiple_clients
+    "${CMAKE_CURRENT_BINARY_DIR}/benchmark.sh"
+    10
+    100
+    one_server_multiple_clients_results_for_client
+    one_server_multiple_clients_results_for_server
+    )
+
+  add_custom_target(
+    memory_benchmark_one_server_multiple_clients_heavy_load
+    "${CMAKE_CURRENT_BINARY_DIR}/benchmark.sh"
+    20
+    100
+    one_server_multiple_clients_heavy_load_results_for_client
+    one_server_multiple_clients_heavy_load_results_for_server
+    )
+
+  add_custom_target(
+    memory_benchmarks
+    DEPENDS
+    memory_benchmark_one_server_one_client
+    memory_benchmark_one_server_multiple_clients
+    memory_benchmark_one_server_multiple_clients_heavy_load
+    )
+
+endif()

=== added file 'benchmarks/memory/benchmark.sh.in'
--- benchmarks/memory/benchmark.sh.in	1970-01-01 00:00:00 +0000
+++ benchmarks/memory/benchmark.sh.in	2012-11-06 12:08:10 +0000
@@ -0,0 +1,18 @@
+#!/bin/bash
+
+# Starting the server
+server_fn=@CMAKE_CURRENT_BINARY_DIR@/massif_$4
+@VALGRIND_EXECUTABLE@ --tool=massif --pages-as-heap=yes --massif-out-file=`echo $server_fn`.out @CMAKE_INSTALL_PREFIX@/bin/mir -f @MEMORY_BENCHMARKS_SOCKET@ &
+server_pid=$!
+
+sleep 5
+
+# Starting the clients
+fn=@CMAKE_CURRENT_BINARY_DIR@/massif_$3
+seq 1 $1 | xargs -I {} -n 1 -P $1 @VALGRIND_EXECUTABLE@ --tool=massif --pages-as-heap=yes --massif-out-file=`echo $fn`.out.{} @CMAKE_INSTALL_PREFIX@/bin/mir_demo_client -f @MEMORY_BENCHMARKS_SOCKET@ -c $2
+
+kill $server_pid
+
+# This is extremely ugly, but we need to introduce some latency to account
+# for the fact that we fire up multiple mir instances in rapid succession.
+sleep 5

=== modified file 'src/client/demo_client.c'
--- src/client/demo_client.c	2012-10-16 23:06:19 +0000
+++ src/client/demo_client.c	2012-11-06 12:08:10 +0000
@@ -22,10 +22,12 @@
 #include <signal.h>
 #include <string.h>
 #include <stdio.h>
+#include <stdlib.h>
 #include <unistd.h>
 #include <getopt.h>
 
 static char const *socket_file = "/tmp/mir_socket";
+static int buffer_swap_count = 0;
 static MirConnection *connection = 0;
 static MirSurface *surface = 0;
 
@@ -41,6 +43,12 @@
     surface = new_surface;
 }
 
+static void surface_next_buffer_callback(MirSurface* surface, void *context)
+{
+    (void) surface;
+    (void) context;
+}
+
 static void surface_release_callback(MirSurface *old_surface, void *context)
 {
     (void)old_surface;
@@ -52,22 +60,25 @@
 {
     int arg;
     opterr = 0;
-    while ((arg = getopt (argc, argv, "hf:")) != -1)
+    while ((arg = getopt (argc, argv, "c:hf:")) != -1)
     {
         switch (arg)
         {
-        case 'f':
-            socket_file = optarg;
-            break;
-
-        case '?':
-        case 'h':
-        default:
-            puts(argv[0]);
-            puts("Usage:");
-            puts("    -f <socket filename>");
-            puts("    -h: this help text");
-            return -1;
+            case 'c':
+                buffer_swap_count = atoi(optarg);
+                break;
+            case 'f':
+                socket_file = optarg;
+                break;
+                
+            case '?':
+            case 'h':
+            default:
+                puts(argv[0]);
+                puts("Usage:");
+                puts("    -f <socket filename>");
+                puts("    -h: this help text");
+                return -1;
         }
     }
 
@@ -111,6 +122,11 @@
     assert(0 <= buffer_package.data_items);
     assert(0 <= buffer_package.fd_items);
 
+    for(int i = 0; i < buffer_swap_count; i++)
+    {
+        mir_wait_for(mir_surface_next_buffer(surface, surface_next_buffer_callback, 0));
+    }
+    
     mir_wait_for(mir_surface_release(surface, surface_release_callback, 0));
     puts("Surface released");
 

